#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup() {
	
	ofBackground(0, 0, 0);
	ofSetLogLevel(OF_LOG_NOTICE);
	ofSetVerticalSync(true);
	
	// Box2d
	box2d.init();
	box2d.setGravity(0, 20);
	box2d.createFloor();
	box2d.setFPS(30.0);
	
	box2d.setContactListener(this);
	
	
	// Sound FX
	sfx.loadSound("sounds/synth.wav");
	
	
}


//--------------------------------------------------------------
void testApp::makeSound(float x, float y, float scl) {
	float widthStep = ofGetWidth() / scl;
	float spd = scl / widthStep;
	sfx.play();
	sfx.setMultiPlay(true);
	sfx.setSpeed(spd);
}

//--------------------------------------------------------------
void testApp::Add(const b2ContactPoint* p) {
	
	// point of collision
	b2Vec2 pt = p->position;
	pt *= OFX_BOX2D_SCALE;
	
	//
	b2Body* b1 = p->shape1->GetBody();
	b2Body* b2 = p->shape2->GetBody();
	
	Thing* prt1 = (Thing*)b1->GetUserData();
	if(prt1) {
		prt1->bHit = true;
		
		makeSound(pt.x, pt.y, prt1->r);
		
		
	}
	
	Thing* prt2 = (Thing*)b2->GetUserData();
	if(prt2) {
		prt2->bHit = true;
		makeSound(pt.x, pt.y, prt2->r);
		
	}
	
}



//--------------------------------------------------------------
void testApp::update() {
	
	
	box2d.update();	
	
	
}

//--------------------------------------------------------------
void testApp::draw() {
	
	box2d.draw();
	
	for(int i=0; i<musicParticles.size(); i++) {
		musicParticles[i].draw();
		musicParticles[i].thing->bHit = false;
	}
	string info = "Press any key to add a shape\nFPS: "+ofToString(ofGetFrameRate());
	ofSetColor(255, 255, 0);
	ofDrawBitmapString(info, 20, 20);
}


//--------------------------------------------------------------
void testApp::keyPressed(int key) {
	
	MusicParticle mp;
	mp.setPhysics(0.4, 0.5, 0.001);
	mp.setup(box2d.getWorld(), mouseX, mouseY, ofRandom(3, 100));
	musicParticles.push_back(mp);
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ) {
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button) {
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button) {
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button) {
	
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h) {
}

