#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){
	
	ofBackground(20, 20, 20);
	ofSetVerticalSync(true);
	
	bSaving = false;
	z		= -300;
	scale	= 15.0;
	gridW	= 100;
	gridH	= 100;
	grid	= new ofPoint[ gridW*gridH ];
	
	for(int i=0; i<gridW; i++) {
		for(int j=0; j<gridH; j++) {
			
			int index = j*gridW+i;
			float x   =	(j*scale);
			float y   = (i*scale);
			grid[index].set(x, y, 0);
		}
	}
	
	saver.init(10, 20, true);
}

//--------------------------------------------------------------
void testApp::update(){
	
	if(saver.bGoTiling == false) {
		float divx = 200.0 + (mouseX/(float)ofGetWidth()) * 500.0;
		float divy = 200.0 + (mouseY/(float)ofGetHeight()) * 500.0;
		
		for(int i=0; i<gridH; i++) {
			for(int j=0; j<gridW; j++) {
				
				int index = i*gridW+j;
				float x   =	(j*scale);
				float y   = (i*scale);
				
				grid[index].x = x;
				grid[index].y = ofSignedNoise(x/divx, y/divy, ofGetElapsedTimef()) * 45.0;
				grid[index].z = y;
				
			}
		}
	}
}

//--------------------------------------------------------------
void testApp::draw() {
	
	saver.begin();
	
	glPushMatrix();
	glTranslated((ofGetWidth()-gridW*scale)/2, 100, z);
	glRotatef(-45, 1, 0, 0);
	for(int i=0; i<gridH-1; i++) {
		glBegin(GL_QUAD_STRIP);		
		for(int j=0; j<gridW; j++) {
			
			float x		= (j*scale);
			float y		= (i*scale);
			int index1	= i*gridW+j;
			int index2	= (i+1)*gridW+j;
			
			glColor3f(1.0-(grid[index2].y/45.0), 0.0, 1.0);
			glVertex3f(grid[index2].x, grid[index2].y, grid[index2].z);
			glColor3f(1.0-(grid[index1].y/45.0), 0.0, 1.0);			
			glVertex3f(grid[index1].x, grid[index1].y, grid[index1].z);
			
			
		}
		glEnd();
	}
	glPopMatrix();
	
	saver.end();
	
	if(bSaving && saver.bGoTiling == false) {
		printf("--- done saving ---\n");
		bSaving = false;
	}
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
	if(key == ' ') {
		printf("--- saving... ---\n");
		saver.finish("high-res.png", false);
		bSaving = true;
	}
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	z += ofMap(y, 0, ofGetHeight(), -100, 100);
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){
	
}

