#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){
	
	ofSetVerticalSync(true);
	ofBackground(90, 90, 90);
	bOutput = false;
	
	int nParticle = 900;
	particles.assign(nParticle, Particle());
	for(int i=0; i<nParticle; i++) {
		particles[i].init(100, 100);
	}
}

//--------------------------------------------------------------
void testApp::update(){
	for(int i=0; i<particles.size(); i++) {
		particles[i].update();
	}	
}

//--------------------------------------------------------------
void testApp::draw(){
	if(bOutput) {
		cairo.begin("awesome.pdf", ofGetWidth(), ofGetHeight(), OF_OUTPUT_PDF);
	}
	
	// -----------------------
	// rectangle with rec-mode center
	// and rotation
	// -----------------------
	ofSetRectMode(OF_RECTMODE_CENTER);
	float num = 10.0;
	for(int i=0; i<num; i++) {
		float n = (float)i/(num-1.0);
		float w = 120.0-(n*100);
		
		cairo.pushMatrix();				// like gl pushMatrix
		cairo.fill();
		cairo.setColor(25, n*255, 55);
		
		cairo.translate(290, 100);		// translate to the position
		cairo.rotate(n*360.0);			// rotate in degrees
		cairo.rect(0, 0, w, w);			// draw the shape

		cairo.popMatrix();				// pop out
		
	}
	ofSetRectMode(OF_RECTMODE_CORNER);
	
	
	// -----------------------
	// some particles that are
	// grouped as a clipping
	// -----------------------
	cairo.pushClippingGroup();
	for(int i=0; i<particles.size(); i++) {
		particles[i].draw(&cairo);
	}	
	cairo.popClippingGroup();
	
	
	// -----------------------
	// simple little curve
	// -----------------------
	cairo.noFill();
	cairo.setColor(255, 255, 0, 100);
	cairo.curve(160, 100, 240, 270, 330, 240, 360, 500);
	
	
	// -----------------------
	// simple bezier curve
	// -----------------------
	cairo.setColor(255, 0, 0, 100);
	cairo.bezier(70, 270,  100,  200, 120, 260, 180, 270);
	
	// -----------------------
	// triangle
	// -----------------------
	cairo.fill();
	cairo.setColor(244, 2, 100, 100);
	cairo.triangle(400, 110, 410, 50, 420, 110);
	
	
	int nRects = 200;
	for(int i=0; i<nRects; i++) {
		float n = (float)i/((float)nRects-1);
		float x = ofGetWidth()*n;
		float w = ofGetWidth()/nRects;
		float h =60+cos(ofGetElapsedTimef()*(n*10))*(n*50);
		float y = ofGetHeight()-h;
		cairo.fill();
		cairo.setColor(255, n*255, 89);
		cairo.rect(x, y, w, h);
	}

	
	// -----------------------
	// funky worm line
	// -----------------------
	if(lines.size() > 0) {
		cairo.beginShape();
		int numPts = lines.size();
		
		// right side
		for(int i=1; i<numPts; i++) {
			float n = (float)i/((float)numPts-1);
			ofxVec2f v = lines[i]-lines[i-1];
			v.normalize();
			v.perpendicular();
			v *= n*20;
			
			cairo.noFill();
			cairo.setColor(255, n*255, 255);
			
			if(i == 1 || i == numPts-1){
				cairo.vertex(lines[i].x+v.x, lines[i].y+v.y);
			}
			
			cairo.vertex(lines[i].x+v.x, lines[i].y+v.y);
			
		}
		cairo.endShape();
		
		
		// left side
		cairo.beginShape();
		for(int i=1; i<numPts; i++) {
			float n = (float)i/((float)numPts-1);
			ofxVec2f v = lines[i]-lines[i-1];
			v.normalize();
			v.perpendicular();
			v *= n*-20;
			cairo.noFill();
			cairo.setColor(255, n*255, 255);
			
			if(i == 1 || i == numPts-1){
				cairo.vertex(lines[i].x+v.x, lines[i].y+v.y);
			}
			
			cairo.vertex(lines[i].x+v.x, lines[i].y+v.y);
			
		}
		
		cairo.endShape();
		
	}
	
	if(bOutput) {
		bOutput = false;
		cairo.end();
	}
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){
	if(key == ' ') {
		bOutput = true;
	}
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	lines.push_back(ofPoint(x, y));
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){
	
}

